// mssg.hh, user interface (CLI, GUI) class
/****************************************************************************

    C-formula interpreter library forint
    Copyright (C) 2000 Alexey Filin

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
    02111-1307, USA.
              
 ****************************************************************************/

#include <iostream>
#include <strstream>
#include <string>
#include <stdio.h>
#include <float.h>
#include <unistd.h> // usleep()

#include "forint_types.hh"

#ifndef _MSSG_HH_
#define _MSSG_HH_

namespace FInt {

enum colorEnum { fg_default=30, fg_red, fg_green, fg_yellow, fg_blue,
		 fg_magenta, fg_cyan, fg_grey,
		 bg_default=48, bg_red=41, bg_green, bg_yellow, bg_blue,
		 bg_magenta, bg_cyan, bg_grey };

class mssg
{
protected:
  const char* str;  // a parsed expression
  uint size;        // its length
  string u_message; // a message
  string u_str_message; // a string with pointed error
  char* substr_beg; // beginning of a substring an error was found in
  uint  substr_len; // lenght of a substring an error was found in
  int locale;
  uint dbl_precision;
  unsigned long slp; // usleep time -- is used by typewriter modeler
  char* emess;  // string to be print before main error message
  char* wmess;  // string to be print before main warning message


public:

  void init(void);

mssg() { init(); }

int
getLocale(void) const { return locale; }

virtual void
setStr(const char* str_bgn, uint s) { str = str_bgn; size = s; }

// set output precision of doubles
  /* DBL_DIG Number of decimal digits of precision in a double */ // float.h
virtual void
precision(uint s=DBL_DIG) { dbl_precision=s; }

virtual void
setSleep(unsigned long msec) { slp = msec*1000; }

virtual const char*
getStr(void) const { return str; }

virtual uint
getStrSize(void) const { return size; }

virtual const char*
getSubStr(void) const { return substr_beg; }

virtual uint
getSubStrSize(void) const { return substr_len; }

virtual string
getMssg(void) const { return u_message; }

// set premessages for error and warning messages
void
setEPremess(const char* str=0)
{
  if (emess) free(emess);
  if (str) emess = strdup(str);
  else emess = 0;

}
void
setWPremess(const char* str=0)
{
  if (wmess) free(wmess);
  if (str) wmess = strdup(str);
  else wmess = 0;
}

virtual
~mssg() {}

void flsh();

// of course to make one call instead three (universal `mess' instead `err',
// `warn' and `mess') is more convenient and conversely to use three calls is
// more convenient, so `err',`warn' and `mess' were appeared...

// first, initialization call
virtual void err(const char* p, uint s,
		 const char* message, uint force=0);

// the rest err's MUST follow the first call

// the number of characters MUST BE pointed !!!
virtual void err(const char message, uint n, uint force=0);
virtual void err(const double message, uint force=0);
virtual void err(const string message, uint force=0);

// first, initialization call
virtual void warn(const char* p, uint s,
		  const char* message, uint force=0);

// the rest warn's MUST follow the first call

// the number of characters MUST BE pointed !!!
virtual void warn(const char message, uint n, uint force=0);
virtual void warn(const double message, uint force=0);
virtual void warn(const string message, uint force=0);

// first, initialization call
virtual void mess(int s, const char* message,
		  colorEnum c1=fg_default, colorEnum c2=bg_default,
		  bool bold=false);

// the rest mess's MUST follow the first call

// the number of characters MUST BE pointed !!!
virtual void mess(const char message, uint n,
		  colorEnum c1=fg_default, colorEnum c2=bg_default,
		  bool bold=false);
virtual void mess(const double message,
		  colorEnum c1=fg_default, colorEnum c2=bg_default,
		  bool bold=false);
virtual void mess(const string message,
		  colorEnum c1=fg_default, colorEnum c2=bg_default,
		  bool bold=false);
};

/*****************************************************************************
*/

class cmssg : public mssg {

  string string_str;

public:
cmssg() : mssg() {}

virtual void
setStr(const char* str_bgn, uint s)
{
  mssg::setStr(str_bgn,s);
  string_str = string (str_bgn,s);
}

virtual void err(const char* p, uint s,
		 const char* message, uint force=0);

// the number of characters MUST BE pointed !!!
virtual void err(const char message, uint n, uint force=0);
virtual void err(const double message, uint force=0);
virtual void err(const string message, uint force=0);

virtual void warn(const char* p, uint s,
		  const char* message, uint force=0);

// the number of characters MUST BE pointed !!!
virtual void warn(const char message, uint n, uint force=0);
virtual void warn(const double message, uint force=0);
virtual void warn(const string message, uint force=0);

virtual void mess(int s, const char* message,
		  colorEnum c1=fg_default, colorEnum c2=bg_default,
		  bool bold=false);

// the number of characters MUST BE pointed !!!
virtual void mess(const char message, uint n,
		  colorEnum c1=fg_default, colorEnum c2=bg_default,
		  bool bold=false);
virtual void mess(const double message,
		  colorEnum c1=fg_default, colorEnum c2=bg_default,
		  bool bold=false);
virtual void mess(const string message,
		  colorEnum c1=fg_default, colorEnum c2=bg_default,
		  bool bold=false);

};

}

#endif // _MSSG_HH_
